function [ T, DTDR , DTDH, IO ] = traveltime(DIST,PHASE,USEPHASE,h,HS,VARR, NPHASE, NLAYER, PHASENAME)

%function [ T, DTDR , DTDH, IO ] = traveltime(DIST,h,HS,VARR)
%
% Calculate travel times
% Input
%    DIST      - array of epicentral distances
%    PHASE     - array of observed phases correspondng to distances
%    h         - source depth
%    HS        - array of layer thicknesses
%    VARR      - array of laeyr velocities
%    NPHASE    - number of phases in model
%    NLAYER    - number of layers in model
%    PHASENAME - phases supported in model
% Output
%    T    - array of distances corresponding to DIST
%    DTDR - array of dT/dx for each distance
%    DTDH - array of dT/dh for each distance
%%%%%

NDIST=length(DIST);
	
	

for ii=1:NDIST

	if( USEPHASE(ii) == 1)
		%%%%%
		%	find the velocity model corresponding to the phase
		%       PHASENAME has H P S etc - first entry layer thickness
		%%%%%
		for i=2:NPHASE
			if(strcmp(PHASE(ii),PHASENAME(i)))
				PhaseIndex = i-1;
				break;
			end
		end
%	PHASE(ii)
%	PHASENAME(PhaseIndex)
%	PhaseIndex
		%%%%%
		% copy the column of the model into the VP array for computations
		%%%%%
		VP=VARR(:,PhaseIndex);
%size(VARR)
%size(VP)
	% here do the phase if it is used
	        x = DIST(ii) ;
		
		ZTOP(1) = 0;
		for i=1:NLAYER
			ZBOT(i) = ZTOP(i) + HS(i) ;
		        if ( i < NLAYER )
				ZTOP(i+1) = ZBOT(i) ;
			end
		end
		
		% find the layer in which the source lies
		NSRC=-1;
		for i=1:NLAYER
			if ( h >= ZTOP(i) && h <= ZBOT(i))
				NSRC=i ;
				HREFR = h - ZTOP(i) ;
				VSRC = VP(i) ;
				break ;
			end
		end
		if NSRC < 1
			NSRC=NLAYER;
			VSRC=VP(NSRC);
			HREFR = 0;
		end
		
		TIMER = 1.0e+38;
		
		% cycle through the possible refractions
		% outer layer to select refractor, inner to compute
		
		for k=NSRC+1:NLAYER
			if(VP(k) > VP(NSRC) )
		
				p = 1./VP(k) ;
				% initialize
				MINDIST= 0.0;
				TIME = p*x ;
				for i=1:k-1
						if( i < NSRC)
							TIME += HS(i) * sqrt(1.-p*p*VP(i)*VP(i))/VP(i) ;
							MINDIST += HS(i) *p*VP(i)/sqrt(1.-p*p*VP(i)*VP(i)) ;
						elseif ( i == NSRC)
							TIME += (2.*HS(i) -(h-ZTOP(i))) * sqrt(1.-p*p*VP(i)*VP(i))/VP(i) ;
							MINDIST += (2.*HS(i) -(h-ZTOP(i))) *p*VP(i)/sqrt(1.-p*p*VP(i)*VP(i)) ;
							DTDRr = p ;
							DTDHr = - sqrt(1.-p*p*VP(i)*VP(i))/VP(i) ;
						else
							TIME += 2.*HS(i) * sqrt(1.-p*p*VP(i)*VP(i))/VP(i) ;
							MINDIST += 2.*HS(i) *p*VP(i)/sqrt(1.-p*p*VP(i)*VP(i)) ;
						end
				end
				% only accept at distances for which the refraction in valid and if 
				% it is earliest arrival
		%printf("k=%d TIME=%f MINDIST=%f DTDR=%f DTDH=%f\n",k,TIME,MINDIST,DTDRr,DTDHr);
				if (TIME < TIMER  && x >= MINDIST)
					TIMER= TIME ;
					DTDRR = DTDRr ;
					DTDHR = DTDHr ;
				end
			end
		end
		
		% now process for upwardly propagating direct arrival
		% note if there is a LVZ, the ray may not get to the station
		
		% the range in ray parameters is p=0 to 1/V(NSRC) if the ray gets to the surface
		
		PMIN=0.0 ;
		PMAX=0.9999 /VP(NSRC) ;
		NP=100 ;
		DP=(PMAX -PMIN)/NP;
		
		% do a crude search
		xold = 0 ;
		pold = 0 ;
		dohalve=0;
		for i=1:NP
			p = PMIN + i*DP ;
			dist = 0.0 ;
			TIME = 0.0 ;
		
			for i=1:NSRC
				if( i < NSRC)
					dist += HS(i) *p*VP(i)/sqrt(1.-p*p*VP(i)*VP(i)) ;
				else
					dist += (h - ZTOP(i)) *p*VP(i)/sqrt(1.-p*p*VP(i)*VP(i)) ;
				end
			end
				if( x >= xold && x <= dist)
					% do interval halving
					dohalve=1;
					break ;
				end
			xold = dist ;
			pold = p ;
		end
		
		% test to see if we found the distance
		% if we did not then treat the arrival as apseudo-refraction
		%%%%%
		% use interval halving to improve
		if (dohalve == 1)
			pleft = pold;
			pright = p ;
			xleft = xold ;
			xright = dist ;
			for int=1:20
				p = 0.5*(pleft + pright);
				TIME = 0;
				dist = 0 ;
				DXDPD = 0;
				DXDHD = 0;
				DTDPD = 0;
				for i=1:NSRC
					if( i < NSRC)
						cosp = sqrt(1.-p*p*VP(i)*VP(i));
						sinp = p*VP(i) ;
						TIME += HS(i) /(cosp*VP(i) ) ;
						dist += HS(i) * sinp/cosp ;
						DTDPD += HS(i)*sinp/(cosp*cosp*cosp) ;
						DXDPD += HS(i)*VP(i)/(cosp*cosp*cosp) ;
						
					else
						cosp = sqrt(1.-p*p*VP(i)*VP(i));
						sinp = p*VP(i) ;
						TIME += (h - ZTOP(i))  /(cosp*VP(i) ) ;
						dist += (h - ZTOP(i))*sinp/cosp ;
						DXDHD = sinp/cosp;
						DTDHD = 1./(cosp*VP(i)) ;
						DTDPD += (h-ZTOP(i))*sinp/(cosp*cosp*cosp) ;
						DXDPD += (h - ZTOP(i))*VP(i)/(cosp*cosp*cosp) ;
					end
				end
			        % exchange left and right depending which is closer to the desired x
				if( dist < x)
					pleft = p ;
					xleft = dist ;
				else
					pright = p ;
					xright = dist ;
				end
			end
			%	printf("Bracketed x=%f [%f,%f] t=%f p=[%f,%f] \n",x,xleft,xright,TIME,pleft,pright);
			TIMED=TIME ;
			PD=0.5*(pleft + pright);
			DTDRD = PD;
		else
			% treat the arrival as a pseudo-dorefraction with source
			% at the layer boundary
			p= 1./VP(NSRC);
			TIMED=p*x;
			for i=1:NSRC-1
				TIMED += HS(i) * sqrt(1.-p*p*VP(i)*VP(i))/VP(i) ;
			end
			DTDRD = p;
			DTDHD = 0.0;
			DTDPD = 0; DXDHD = 0; DXDPD = 1;
		end
		
		% end of direct
		%printf("%f %f %f %f\n",x,TIMED,TIMER,MINDIST);
		
		if ( TIMED < TIMER)
			T(ii)    = TIMED;
			DTDR(ii) = DTDRD ;
			DTDH(ii) = DTDHD - DTDPD*DXDHD/DXDPD ;
			IO(ii) = 180. - asind(DTDR(ii)*VSRC);
		else
			T(ii)    = TIMER  ;
			DTDR(ii) = DTDRR  ;
			DTDH(ii) = DTDHR  ;
			IO(ii) =  asind(DTDR(ii)*VSRC);
		end

	else
		T(ii)    = 0.0  ;
		DTDR(ii) = 0.0  ;
		DTDH(ii) = 0.0  ;
		IO(ii) =   0.0 ;
	end
end	
	
endfunction
